// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
   "cloudera/Util",
   "cloudera/common/Humanize",
   "cloudera/common/TimeUtil",
   "cloudera/chart/TimeRange",
   "cloudera/chart/TimeControl2"
], function(Util, Humanize, TimeUtil, TimeRange, TimeControl2) {
  var zoomInBtnSuffix = "_zoomIn";
  var zoomOutBtnSuffix = "_zoomOut";
  var dateSelectorBtnSuffix = "_dateSelector";
  var customDateDialogSuffix = "_customDateDialog";
  var startDateInputSuffix = "_customDateDialog_startDate";
  var endDateInputSuffix = "_customDateDialog_endDate";
  var applyBtnSuffix = "_customDateDialog_apply";
  var errorSuffix = "_customDateDialog_error";

  var buildTimeControl = function(id) {
    var $container = $("<div>").attr("id", id);

    var $zoomOutBtn = $("<a>")
    .attr("href", "#")
    .attr("id", id + zoomOutBtnSuffix)
    .text("Zoom Out");

    var $zoomInBtn = $("<a>")
    .attr("href", "#")
    .attr("id", id + zoomInBtnSuffix)
    .text("Zoom In");

    var $customBtn = $("<a>")
    .attr("href", "#")
    .attr("id", id + dateSelectorBtnSuffix)
    .text("Custom");

    var $customDialog = $("<div>")
    .attr("id", id + customDateDialogSuffix)
    .css("display", "none");

    var $fromInput = $("<input>")
    .attr("id", id + startDateInputSuffix)
    .attr("type", "text");

    var $toInput = $("<input>")
    .attr("id", id + endDateInputSuffix)
    .attr("type", "text");

    var $applyBtn = $("<button>")
    .attr("id", id + applyBtnSuffix)
    .text("Apply");

    var $errorDiv = $("<div>")
    .attr("id", id + errorSuffix);

    var presets = [30, 60, 90, 360, 720, 1440];
    var presetIds = ["30m", "60m", "90m", "6h", "12h", "24h"];
    var i;

    for (i = 0; i < presetIds.length; i += 1) {
      var $presetBtn = $("<button>")
      .attr("id", id + "_" + presetIds[i])
      .text(presetIds[i]);
      $customDialog.append($presetBtn);
    }

    $customDialog
    .append($fromInput)
    .append($toInput)
    .append($applyBtn)
    .append($errorDiv);

    $container.append($zoomOutBtn)
    .append($zoomInBtn)
    .append($customBtn)
    .append($customDialog);

    return $container;
  };

  var removeTimeControl = function(id) {
    $("#" + id).remove();
  };

  describe("TimeControl2 Tests", function() {
    var id="tc",
    options = {
      mode: "INTERACTIVE",
      showRange: true,
      firstDate: new Date(1),
      lastDate: new Date(10),
      firstVisibleDate: new Date(5),
      lastVisibleDate: new Date(10),
      markerDate: new Date(7),
      isCurrentMode: false
    }, tc;

    beforeEach(function() {
      jasmine.Ajax.useMock();
      spyOn(TimeUtil, 'getTimezoneDelta').andReturn(0);

      $("body").append(buildTimeControl(id));
      tc = new TimeControl2(options, id);
      spyOn(tc.state, "zoomIn").andCallThrough();
      spyOn(tc.state, "zoomOut").andCallThrough();
      spyOn(tc.marker, "onWindowResized").andCallThrough();
      spyOn(tc.state, "moveToNow").andCallThrough();
      spyOn(tc.state, "expandRange").andCallThrough();
      spyOn(tc.state, "setSelectedRange").andCallThrough();

      tc.openCustomDateDialog = function(options) {
        $("#" + id + customDateDialogSuffix).show();
      };

      tc.closeCustomDateDialog = function() {
        $("#" + id + customDateDialogSuffix).hide();
      };

      var customServerTimeRange;

      tc.getCustomServerTimeRange = function() {
        return customServerTimeRange;
      };

      tc.setCustomServerTimeRange = function(range) {
        customServerTimeRange = range;
      };
    });

    afterEach(function() {
      $("#" + id).remove();
      if (tc) {
        tc.unsubscribe();
      }
    });

    it("should trigger the zoomIn operation when clicking on the zoom in button", function() {
      $("#" + id + zoomInBtnSuffix).trigger("click");
      expect(tc.state.zoomIn).toHaveBeenCalled();
    });

    it("should trigger the zoom out operation when clicked on the zoom out button", function() {
      $("#" + id + zoomOutBtnSuffix).trigger("click");
      expect(tc.state.zoomOut).toHaveBeenCalled();
    });

    it("should trigger the expand operation when the expandTimeRangeSelection event is fired", function() {
      $.publish("expandTimeRangeSelection");
      expect(tc.state.expandRange).toHaveBeenCalled();
    });

    it("should trigger the now operation when the switchToCurrent event is fired", function() {
      $.publish("switchToCurrent");
      expect(tc.state.moveToNow).toHaveBeenCalled();
    });

    it("should make the custom date dialog appear when the custom date icon is clicked", function() {
      $("#" + id + dateSelectorBtnSuffix).trigger("click");
      expect($("#" + id + customDateDialogSuffix).is(":visible")).toEqual(true);
    });

    it("should hide the custom date dialog when the OK button is clicked", function() {
      $("#" + id + dateSelectorBtnSuffix).trigger("click");
      expect($("#" + id + customDateDialogSuffix).is(":visible")).toEqual(true);

      $("#" + id + applyBtnSuffix).trigger("click");
      expect($("#" + id + customDateDialogSuffix).is(":visible")).toEqual(false);
      expect(tc.state.setSelectedRange).toHaveBeenCalled();
    });

    it("should make the custom date dialog appear again when the custom date icon is clicked again", function() {
      $("#" + id + dateSelectorBtnSuffix).trigger("click");
      expect($("#" + id + customDateDialogSuffix).is(":visible")).toEqual(true);
    });
  });

  describe("TimeControl2 Tests with showRange=false, showMarker=true", function() {
    var id="tc",
    options = {
      mode: "INTERACTIVE",
      showRange: false,
      showMarker: true,
      firstDate: new Date(1),
      lastDate: new Date(10),
      firstVisibleDate: new Date(5),
      lastVisibleDate: new Date(10),
      markerDate: new Date(7),
      isCurrentMode: false
    }, tc;

    beforeEach(function() {
      jasmine.Ajax.useMock();
      spyOn(TimeUtil, 'getTimezoneDelta').andReturn(0);

      $("body").append(buildTimeControl(id));
      tc = new TimeControl2(options, id);
      spyOn(tc.state, "zoomIn").andCallThrough();
      spyOn(tc.state, "zoomOut").andCallThrough();
      spyOn(tc.marker, "onWindowResized").andCallThrough();
      spyOn(tc.state, "moveToNow").andCallThrough();
      spyOn(tc.state, "expandRange").andCallThrough();
      spyOn(tc.state, "setMarkerDate").andCallThrough();

      tc.openCustomDateDialog = function(options) {
        $("#" + id + customDateDialogSuffix).show();
      };

      tc.closeCustomDateDialog = function() {
        $("#" + id + customDateDialogSuffix).hide();
      };

      var customServerMarkerDate;

      tc.getCustomServerMarkerDate = function() {
        return customServerMarkerDate;
      };

      tc.setCustomServerMarkerDate = function(date) {
        customServerMarkerDate = date;
      };
    });

    afterEach(function() {
      $("#" + id).remove();
      if (tc) {
        tc.unsubscribe();
      }
    });

    it("should trigger the zoomIn operation when clicking on the zoom in button", function() {
      $("#" + id + zoomInBtnSuffix).trigger("click");
      expect(tc.state.zoomIn).toHaveBeenCalled();
    });

    it("should trigger the zoom out operation when clicked on the zoom out button", function() {
      $("#" + id + zoomOutBtnSuffix).trigger("click");
      expect(tc.state.zoomOut).toHaveBeenCalled();
    });

    it("should trigger the expand operation when the expandTimeRangeSelection event is fired", function() {
      $.publish("expandTimeRangeSelection");
      expect(tc.state.expandRange).toHaveBeenCalled();
    });

    it("should trigger the now operation when the switchToCurrent event is fired", function() {
      $.publish("switchToCurrent");
      expect(tc.state.moveToNow).toHaveBeenCalled();
    });

    it("should make the custom date dialog appear when the custom date icon is clicked", function() {
      $("#" + id + dateSelectorBtnSuffix).trigger("click");
      expect($("#" + id + customDateDialogSuffix).is(":visible")).toEqual(true);
    });

    it("should hide the custom date dialog when the OK button is clicked", function() {
      $("#" + id + dateSelectorBtnSuffix).trigger("click");
      expect($("#" + id + customDateDialogSuffix).is(":visible")).toEqual(true);

      $("#" + id + applyBtnSuffix).trigger("click");
      expect($("#" + id + customDateDialogSuffix).is(":visible")).toEqual(false);
      expect(tc.state.setMarkerDate).toHaveBeenCalled();
    });

    it("should make the custom date dialog appear again when the custom date icon is clicked again", function() {
      $("#" + id + dateSelectorBtnSuffix).trigger("click");
      expect($("#" + id + customDateDialogSuffix).is(":visible")).toEqual(true);
    });
  });
});
