// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
  'cloudera/cmf/charts/ChartsSearch'
], function(ChartsSearch) {
  describe("ChartsSearch Tests", function() {
    var module, id = "chartsSearchContainer", options = {
      container: "#" + id,
      recentUri: "dontcare",
      searchUri: "dontcare",
      metric: "",
      tsquery: ""
    }, SOME_TSQUERY = "SELECT Metric FOO", SOME_METRIC = "Metric FOO";

    beforeEach(function() {
      jasmine.Ajax.useMock();
      clearAjaxRequests();
      $('<div id="' + id + '">' +
       '<div class="control-group">' +
        '<input class="metricInput" type="text"/>' +
        '</div>' +
        '<div class="control-group">' +
        '<input class="tsqueryInput" data-bind="value: tsquery" type="hidden"/>' +
        '</div>' +
        '</div>').appendTo(document.body);
      spyOn($, "publish");
    });

    afterEach(function() {
      $("#" + id).remove();
    });

    it("should show error class on metric input field when empty", function() {
      module = new ChartsSearch(options);
      expect(module.searchMode()).toEqual("basic");

      module.handleClick();
      expect($(".metricInput").closest(".control-group.error").length).toBeGreaterThan(0);
      expect($.publish).wasNotCalled();
    });

    it("should not show error class on metric input field not empty", function() {
      module = new ChartsSearch(options);
      expect(module.searchMode()).toEqual("basic");
      $(".metricInput").val("Metric FOO");
      module.tsquery('select Metric FOO');

      module.handleClick();
      expect($(".metricInput").closest(".control-group.error").length).toEqual(0);
      expect($.publish).wasCalled();
    });

    it("should not show error class on tsquery input fields when it is empty", function() {
      module = new ChartsSearch(options);
      module.searchMode("advanced");
      // Test Binding.
      module.tsquery("");
      expect($(".tsqueryInput").val()).toEqual("");

      module.handleClick();
      expect($(".tsqueryInput").closest(".control-group.error").length).toBeGreaterThan(0);
      expect($.publish).wasNotCalled();
    });

    it("should not show error class on tsquery input fields when it is not empty", function() {
      module = new ChartsSearch(options);
      module.searchMode("advanced");

      module.tsquery(SOME_TSQUERY + "1");
      expect($(".tsqueryInput").val()).toEqual(SOME_TSQUERY + "1");

      module.handleClick();
      expect($(".tsqueryInput").closest(".control-group.error").length).toEqual(0);
      expect($.publish).wasCalledWith("tsqueryChanged", [SOME_TSQUERY + "1"]);
    });

    it("should trigger a $.publish", function() {
      module = new ChartsSearch(options);
      module.searchMode("advanced");

      $(".tsqueryInput").val(SOME_TSQUERY + "2");

      // Added to the recent list.
      module.handleClick();
      expect($.publish).wasCalledWith("tsqueryChanged", [SOME_TSQUERY + "2"]);
      expect(module.recentList().length).toEqual(1);

      // Added to the recent list.
      $(".tsqueryInput").val(SOME_TSQUERY + "3");
      module.handleClick();
      expect(module.recentList().length).toEqual(2);

      // No change.
      $(".tsqueryInput").val(SOME_TSQUERY + "3");
      module.handleClick();
      expect(module.recentList().length).toEqual(2);
    });

    it("should generate context key values", function() {
      module = new ChartsSearch($.extend({}, options, {
        context: {
          '$foo': "FOO",
          '$bar': "BAR"
        }
      }));
      expect(module.contextKeyValues().length).toEqual(2);
      expect(module.contextKeyValues()[0].name).toEqual("$foo");
      expect(module.contextKeyValues()[0].value).toEqual("FOO");
    });

    it('should validate when the basic query has no associated tsquery', function() {
      module = new ChartsSearch(options);
      // Add something to basic query box.
      $('.metricInput').val('catpants');
      module.tsquery('');
      expect(module.validateBasic()).toBeFalsy();
    });

    it('should blank out tsquery input when no typeahead results received in basic', function() {
      module = new ChartsSearch(options);
      // Set some fake values as the tsquery.
      $('.tsqueryInput').val('select catpants');
      module.tsquery('tsquery catpants');

      spyOn(module, "handleClick");
      // Make sure to trigger our typeahead.
      $('.metricInput').val('catpants').trigger('keyup');
      var request = mostRecentAjaxRequest();
      request.response({
        status: 200,
        responseText: '[]'
      });
      expect($('.tsqueryInput').val()).toEqual('');
      expect(module.tsquery()).toEqual('');
      expect(module.handleClick).wasNotCalled();
    });

    it('should blank out tsquery input when no typeahead results received in basic', function() {
      module = new ChartsSearch(options);
      // Set some fake values as the tsquery.
      $('.tsqueryInput').val('select catpants');
      module.tsquery('tsquery catpants');

      spyOn(module, "handleClick");
      // Make sure to trigger our typeahead.
      $('.metricInput').val('catpants').trigger('keyup');
      var request = mostRecentAjaxRequest();
      request.response({
        status: 200,
        responseText: JSON.stringify([{
          label: "catpants",
          value: {
            tsquery: "SELECT some Query"
          }
        }])
      });
      // Simulate the enter key.
      var keyupEvt = $.Event('keyup');
      keyupEvt.keyCode = 13;
      $('.metricInput').trigger(keyupEvt);

      expect(module.handleClick).wasCalled();
      expect(module.tsquery()).toEqual('SELECT some Query');
    });

    it('should call search on enter', function() {
      module = new ChartsSearch(options);
      spyOn(module, "handleClick");

      // Set some fake values as the tsquery.
      var event = $.Event('keyup');
      event.which = $.ui.keyCode.ENTER; // The Enter key.
      $('.tsqueryInput').val('select catpants').trigger(event);
      expect(module.handleClick).wasCalled();
    });
  });
});
